#include "resmes.hpp"
#include "log.hpp"

#include <iomanip>

#include <sys/resource.h>

using namespace std;

/**
 * The resources measured in this program.
 */
map<string, resmes> resources;

resmes::resmes(): depth(0)
{
}

void resmes::set_desc(const string &d)
{
   desc = d;
}

void resmes::start()
{
   depth++;

   if(depth > 1)
      return;

   rusage temp;

   int ret = getrusage(RUSAGE_SELF, &temp);
   if(ret == -1)
      lerr(L_ERR) << "Timing malfunction" << endl;
   
   start_time.tv_sec = temp.ru_utime.tv_sec + temp.ru_stime.tv_sec;
   start_time.tv_usec = temp.ru_utime.tv_usec + temp.ru_stime.tv_usec;

   if(start_time.tv_usec >= 1000000)
   {
      start_time.tv_sec++;
      start_time.tv_usec -= 1000000;
   }
}

void resmes::stop()
{
   if(depth == 0)
      return;

   depth--;

   if(depth == 0)
   {
      rusage stop_time;
      int ret = getrusage(RUSAGE_SELF, &stop_time);
      if(ret == -1)
	 lerr(L_ERR) << "Timing malfunction" << endl;
      
      timeval difftime;
      difftime.tv_sec = stop_time.ru_utime.tv_sec + stop_time.ru_stime.tv_sec
	 - start_time.tv_sec;
      difftime.tv_usec = stop_time.ru_utime.tv_usec + stop_time.ru_stime.tv_usec
	  - start_time.tv_usec;
      
      if(difftime.tv_usec < 0)
      {
 	 difftime.tv_sec--;
 	 difftime.tv_usec += 1000000;
      }
      
      periods.push_back(difftime.tv_sec + difftime.tv_usec / 1000000.0);
   }
}

ostream &operator<<(ostream &os, const resmes &res)
{
   double total_time = 0.0;
   for(size_t i = 0; i < res.periods.size(); i++)
      total_time += res.periods[i];

   os << res.desc << " (" << res.periods.size() << "): " << setprecision(3) << total_time << " s" << endl;

   return os;
}
